@@ -1,16 +1,21 @@
-import contextlib
 import logging
+import shutil
 from copy import deepcopy
-from typing import Dict, Any
+from typing import Dict, Any, Type
 
 import h5py
 from Bio import SeqIO
-from pandas import read_csv
+from pandas import read_csv, DataFrame
 from tqdm import tqdm
 
-from bio_embeddings.embed import EmbedderInterface
-from bio_embeddings.embed.albert import AlbertEmbedder
-from bio_embeddings.embed.seqvec.SeqVecEmbedder import SeqVecEmbedder
+from bio_embeddings.embed import (
+    ProtTransAlbertBFDEmbedder,
+    ProtTransBertBFDEmbedder,
+    EmbedderInterface,
+    SeqVecEmbedder,
+    ProtTransXLNetUniRef100Embedder,
+    UniRepEmbedder,
+)
 from bio_embeddings.utilities import (
     InvalidParameterError,
     get_model_file,
@@ -19,11 +24,63 @@ from bio_embeddings.utilities import (
     get_model_directories_from_zip,
     FileManagerInterface,
 )
+from bio_embeddings.utilities.backports import nullcontext
 
 logger = logging.getLogger(__name__)
 
 
-def _get_reduced_embeddings_file_context(file_manager: FileManagerInterface, result_kwargs: Dict):
+def _print_expected_file_sizes(
+    embedder: EmbedderInterface, mapping_file: DataFrame, result_kwargs: Dict[str, Any]
+) -> None:
+    """
+    Logs the lower bound size of embeddings_file and reduced_embedding_file
+
+    :param embedder: the embedder being used
+    :param mapping_file: the mapping file of the sequences
+    :param result_kwargs: the kwargs passed to the pipeline --> will decide what to print
+
+    :return: Nothing.
+    """
+    per_amino_acid_size_in_bytes = 4 * embedder.embedding_dimension * embedder.number_of_layers
+    per_protein_size_in_bytes = 4 * embedder.embedding_dimension
+
+    total_number_of_proteins = len(mapping_file)
+    total_aa = mapping_file['sequence_length'].sum()
+
+    embeddings_file_size_in_MB = per_amino_acid_size_in_bytes * total_aa * pow(10, -6)
+    reduced_embeddings_file_size_in_MB = per_protein_size_in_bytes * total_number_of_proteins * pow(10, -6)
+
+    required_space_in_MB = 0
+
+    if result_kwargs.get("reduce") is True:
+        logger.info(f"The minimum expected size for the reduced_embedding_file is "
+                    f"{reduced_embeddings_file_size_in_MB:.3f}MB.")
+
+        required_space_in_MB += reduced_embeddings_file_size_in_MB
+
+    if not (result_kwargs.get("reduce") is True and result_kwargs.get("discard_per_amino_acid_embeddings") is True):
+        logger.info(f"The minimum expected size for the embedding_file is {embeddings_file_size_in_MB:.3f}MB.")
+
+        required_space_in_MB += embeddings_file_size_in_MB
+
+    _, _, available_space_in_bytes = shutil.disk_usage(result_kwargs.get('prefix'))
+
+    available_space_in_MB = available_space_in_bytes * pow(10, -6)
+
+    if available_space_in_MB < required_space_in_MB:
+        logger.warning(f"You are attempting to generate {required_space_in_MB:.3f}MB worth of embeddings, "
+                       f"but only {available_space_in_MB:.3f}MB are available at "
+                       f"the prefix({result_kwargs.get('prefix')}). \n"
+                       f"We suggest you stop execution NOW and double check you have enough free space available. "
+                       f"Alternatively, try reducing the input FASTA file.")
+    else:
+        logger.info(f"You are going to generate a total of {required_space_in_MB:.3f}MB of embeddings, and have "
+                    f"{available_space_in_MB:.3f}MB available at {result_kwargs.get('prefix')}.")
+
+
+def _get_reduced_embeddings_file_context(
+    file_manager: FileManagerInterface, result_kwargs: Dict[str, Any]
+):
     """
     :param file_manager: The FileManager derived class which will be used to create the file
     :param result_kwargs: A dictionary which will be updated in-place to include the path to the newly created file
@@ -32,16 +89,19 @@ def _get_reduced_embeddings_file_context(file_manager: FileManagerInterface, res
     """
 
     # Create reduced embeddings file if set in params
-    result_kwargs.setdefault('reduce', False)
+    result_kwargs.setdefault("reduce", False)
 
-    if result_kwargs['reduce'] is True:
-        reduced_embeddings_file_path = file_manager.create_file(result_kwargs.get('prefix'),
-                                                                result_kwargs.get('stage_name'),
-                                                                'reduced_embeddings_file', extension='.h5')
-        result_kwargs['reduced_embeddings_file'] = reduced_embeddings_file_path
+    if result_kwargs["reduce"] is True:
+        reduced_embeddings_file_path = file_manager.create_file(
+            result_kwargs.get("prefix"),
+            result_kwargs.get("stage_name"),
+            "reduced_embeddings_file",
+            extension=".h5",
+        )
+        result_kwargs["reduced_embeddings_file"] = reduced_embeddings_file_path
         return h5py.File(reduced_embeddings_file_path, "w")
 
-    return contextlib.nullcontext()
+    return nullcontext()
 
 
 def _get_embeddings_file_context(
@@ -61,7 +121,7 @@ def _get_embeddings_file_context(
             raise InvalidParameterError(
                 "Cannot have discard_per_amino_acid_embeddings=True and reduce=False. Both must be True."
             )
-        return contextlib.nullcontext()
+        return nullcontext()
     else:
         embeddings_file_path = file_manager.create_file(
             result_kwargs.get("prefix"),
@@ -78,13 +138,20 @@ def embed_and_write_batched(
     file_manager: FileManagerInterface,
     result_kwargs: Dict[str, Any],
 ) -> Dict[str, Any]:
-    """ The shared code between the SeqVec and the Albert pipeline """
+    """ The shared code between the SeqVec, Albert, Bert and XLNet pipelines """
     # Lazy fasta file reader. The mapping file contains the corresponding ids in the same order
     sequences = (
         str(entry.seq)
         for entry in SeqIO.parse(result_kwargs["remapped_sequences_file"], "fasta")
     )
+    # We want to read the unnamed column 0 as str (esp. with simple_remapping), which requires some workarounds
+    # https://stackoverflow.com/a/29793294/3549270
     mapping_file = read_csv(result_kwargs["mapping_file"], index_col=0)
+    mapping_file.index = mapping_file.index.astype('str')
+
+    # Print the minimum required file sizes
+    _print_expected_file_sizes(embedder, mapping_file, result_kwargs)
+
     # Open embedding files or null contexts and iteratively save embeddings to file
     with _get_embeddings_file_context(
         file_manager, result_kwargs
@@ -92,7 +159,7 @@ def embed_and_write_batched(
         file_manager, result_kwargs
     ) as reduced_embeddings_file:
         embedding_generator = embedder.embed_many(
-            sequences, result_kwargs["max_amino_acids"]
+            sequences, result_kwargs.get("max_amino_acids")
         )
         for sequence_id, embedding in zip(
             mapping_file.index, tqdm(embedding_generator, total=len(mapping_file))
@@ -102,7 +169,7 @@ def embed_and_write_batched(
 
             if result_kwargs.get("reduce") is True:
                 reduced_embeddings_file.create_dataset(
-                    sequence_id, data=embedder.reduce_per_protein(embedding),
+                    sequence_id, data=embedder.reduce_per_protein(embedding)
                 )
     return result_kwargs
 
@@ -115,71 +182,63 @@ def seqvec(**kwargs) -> Dict[str, Any]:
     # Initialize pipeline and model specific options:
     result_kwargs.setdefault("max_amino_acids", 15000)
 
-    if result_kwargs.get("seqvec_version") == 2 or result_kwargs.get("vocabulary_file"):
-        necessary_files.append("vocabulary_file")
-        result_kwargs["seqvec_version"] = 2
-
     # Download necessary files if needed
     for file in necessary_files:
         if not result_kwargs.get(file):
-            file_path = file_manager.create_file(
-                result_kwargs.get("prefix"), result_kwargs.get("stage_name"), file
-            )
-
-            get_model_file(
-                path=file_path,
-                model="seqvecv{}".format(result_kwargs.get("seqvec_version", 1)),
-                file=file,
-            )
-
-            result_kwargs[file] = file_path
+            result_kwargs[file] = get_model_file(model="seqvec", file=file)
 
     embedder = SeqVecEmbedder(**result_kwargs)
     return embed_and_write_batched(embedder, file_manager, result_kwargs)
 
 
-def albert(**kwargs):
-    necessary_directories = ["model_directory"]
+def transformer(
+    embedder_class: Type[EmbedderInterface], max_amino_acids_default: int, **kwargs
+):
     result_kwargs = deepcopy(kwargs)
     file_manager = get_file_manager(**kwargs)
+    result_kwargs.setdefault("max_amino_acids", max_amino_acids_default)
 
-    result_kwargs.setdefault("max_amino_acids", 500)
-
+    necessary_directories = ["model_directory"]
     for directory in necessary_directories:
         if not result_kwargs.get(directory):
-            directory_path = file_manager.create_directory(
-                result_kwargs.get("prefix"), result_kwargs.get("stage_name"), directory
+            result_kwargs[directory] = get_model_directories_from_zip(
+                model=embedder_class.name, directory=directory
             )
 
-            get_model_directories_from_zip(
-                path=directory_path, model="albert", directory=directory
-            )
-
-            result_kwargs[directory] = directory_path
-
-    embedder = AlbertEmbedder(**result_kwargs)
+    embedder = embedder_class(**result_kwargs)
     return embed_and_write_batched(embedder, file_manager, result_kwargs)
 
 
-def fasttext(**kwargs):
-    pass
+def prottrans_albert(**kwargs):
+    return transformer(ProtTransAlbertBFDEmbedder, 3035, **kwargs)
 
 
-def glove(**kwargs):
-    pass
+def prottrans_bert_bfd(**kwargs):
+    return transformer(ProtTransBertBFDEmbedder, 6024, **kwargs)
 
 
-def word2vec(**kwargs):
-    pass
+def prottrans_xlnet(**kwargs):
+    return transformer(ProtTransXLNetUniRef100Embedder, 4000, **kwargs)
+
+
+def unirep(**kwargs) -> Dict[str, Any]:
+    if kwargs.get("use_cpu") is not None:
+        raise InvalidParameterError("UniRep does not support configuring `use_cpu`")
+    result_kwargs = deepcopy(kwargs)
+    file_manager = get_file_manager(**kwargs)
+    embedder = UniRepEmbedder(**result_kwargs)
+    # We don't actually batch with UniRep, but embed_and_write_batched
+    # works anyway since UniRepEmbedder still implements `embed_many`
+    return embed_and_write_batched(embedder, file_manager, result_kwargs)
 
 
 # list of available embedding protocols
 PROTOCOLS = {
     "seqvec": seqvec,
-    "fasttext": fasttext,
-    "glove": glove,
-    "word2vec": word2vec,
-    "albert": albert,
+    "prottrans_albert_bfd": prottrans_albert,
+    "prottrans_bert_bfd": prottrans_bert_bfd,
+    "prottrans_xlnet_uniref100": prottrans_xlnet,
+    "unirep": unirep,
 }
 
 
@@ -200,7 +259,10 @@ def run(**kwargs):
     -------
     Dictionary with results of stage
     """
-    check_required(kwargs, ['protocol', 'prefix', 'stage_name', 'remapped_sequences_file', 'mapping_file'])
+    check_required(
+        kwargs,
+        ["protocol", "prefix", "stage_name", "remapped_sequences_file", "mapping_file"],
+    )
 
     if kwargs["protocol"] not in PROTOCOLS:
         raise InvalidParameterError(
