@@ -1,5 +1,5 @@
 from bio_embeddings.utilities.remote_file_retriever import get_model_file, get_model_directories_from_zip
-from bio_embeddings.utilities.logging import Logger
 from bio_embeddings.utilities.helpers import *
 from bio_embeddings.utilities.exceptions import *
 from bio_embeddings.utilities.filemanagers import get_file_manager
+from bio_embeddings.utilities.filemanagers.FileManagerInterface import FileManagerInterface
