@@ -1,19 +1,24 @@
 import os
 from copy import deepcopy
 from datetime import datetime
+from pathlib import Path
+from typing import Dict, Callable
+
+import importlib_metadata
+
 from bio_embeddings.embed.pipeline import run as run_embed
+from bio_embeddings.extract.pipeline import run as run_extract
 from bio_embeddings.project.pipeline import run as run_project
-from bio_embeddings.visualize.pipeline import run as run_visualize
-# from bio_embeddings.extract_features.pipeline import run as run_extract_features
 from bio_embeddings.utilities import get_file_manager, read_fasta, reindex_sequences, write_fasta_file, \
     check_required, MD5ClashException
 from bio_embeddings.utilities.config import read_config_file, write_config_file
+from bio_embeddings.visualize.pipeline import run as run_visualize
 
 _STAGES = {
     "embed": run_embed,
     "project": run_project,
-    "visualize": run_visualize
-    # "extract_features": run_extract_features
+    "visualize": run_visualize,
+    "extract": run_extract
 }
 
 _IN_CONFIG_NAME = "input_parameters_file"
@@ -49,7 +54,7 @@ def _process_fasta_file(**kwargs):
 
     sequences = read_fasta(kwargs['sequences_file'])
     sequences_file_path = file_manager.create_file(kwargs.get('prefix'), None, 'sequences_file',
-                                                           extension='.fasta')
+                                                   extension='.fasta')
     write_fasta_file(sequences, sequences_file_path)
 
     result_kwargs['sequences_file'] = sequences_file_path
@@ -84,13 +89,15 @@ def _process_fasta_file(**kwargs):
     return result_kwargs
 
 
-def run(config_file_path, **kwargs):
+def _null_function(config: Dict) -> None:
+    pass
 
-    if not _valid_file(config_file_path):
-        raise Exception("No config or invalid config was passed.")
 
-    # read configuration and execute
-    config = read_config_file(config_file_path)
+def execute_pipeline_from_config(config: Dict,
+                                 post_stage: Callable[[Dict], None] = _null_function,
+                                 **kwargs) -> Dict:
+
+    original_config = deepcopy(config)
 
     check_required(
         config,
@@ -120,9 +127,11 @@ def run(config_file_path, **kwargs):
         # create the prefix
         file_manager.create_prefix(prefix)
 
-    # copy config to prefix. Need to re-read because global was popped!
+    Path(prefix).joinpath("bio_embeddings_version.txt").write_text(importlib_metadata.version("bio_embeddings"))
+
+    # Copy original config to prefix
     global_in = file_manager.create_file(prefix, None, _IN_CONFIG_NAME, extension='.yml')
-    write_config_file(global_in, read_config_file(config_file_path))
+    write_config_file(global_in, original_config)
 
     global_parameters = _process_fasta_file(**global_parameters)
 
@@ -178,6 +187,22 @@ def run(config_file_path, **kwargs):
         # Store in global_out config for later retrieval (e.g. depends_on)
         config[stage_name] = stage_output_parameters
 
+        # Execute post-stage function, if provided
+        post_stage(stage_output_parameters)
+
     config['global'] = global_parameters
     global_out = file_manager.create_file(prefix, None, _OUT_CONFIG_NAME, extension='.yml')
     write_config_file(global_out, config)
+
+    return config
+
+
+def parse_config_file_and_execute_run(config_file_path: str, **kwargs):
+
+    if not _valid_file(config_file_path):
+        raise Exception("No config or invalid config was passed.")
+
+    # read configuration and execute
+    config = read_config_file(config_file_path)
+
+    execute_pipeline_from_config(config, **kwargs)
\ No newline at end of file
